
import java.util.Scanner;

/**
 * 本程序主要用于演示将输入输出代码与业务逻辑代码相分离。
 * @author zhrb 
 *
 */
public class StuManagementDemo{

	public static void main(String[] args) {
		Student stu = null;
		Scanner scanner = new Scanner(System.in);
		while(true){
			mainMenu();
			String choice = scanner.nextLine();
			switch (choice) {
			case "1"://糟糕的设计，处理输入输出的代码与业务逻辑相耦合，杂乱无章
				System.out.println("请输入学生的姓名");
				String name = scanner.nextLine();
				System.out.println("请输入学生的成绩");
				int grade = Integer.parseInt(scanner.nextLine());//无容错处理
				stu = new Student(name, grade);
				/*有的同学可能还会往里面放一些其他的业务逻辑代码，比如修改密码、存储学生
				等业务，处理输入输出的代码与业务逻辑代码掺杂在一起，较难理解*/
				System.out.println("学生基本信息为"+stu);
				break;
			case "2":
				stu = addStudentMenu(scanner);
				System.out.println("学生基本信息为"+stu);
				break;
			case "3":
				System.out.println("退出程序");
				scanner.close();
				System.exit(0);
				break;
			default:
				System.out.println("输入错误请重输");
				break;
			}
		}

	}

	
	
	/*
	 * 该方法主要负责：1.处理输入输出。2.调用包含业务逻辑的代码并返回结果。
	 * 不负责具体的业务逻辑代码。
	 */
	private static Student addStudentMenu(Scanner sc) {
		
		System.out.println("请输入学生的姓名");
		String name = sc.nextLine();
		int grade = 0;
		while(true){
			System.out.println("请输入学生的成绩");
			String gradeStr = sc.nextLine();
			try {
				grade = Integer.parseInt(gradeStr);
				break;//如果转换无误则跳出循环
			} catch (NumberFormatException e) {
				System.out.println("请输入数字格式的成绩!");
			}
		}
		return StudentManagement.createStudent(name, grade);//获取输入，然后调用包含业务逻辑的代码
		
	}
	private static void mainMenu() {
		System.out.println("========");
		System.out.println("1.增加学生---糟糕设计(输入、输出与业务逻辑相耦合)");
		System.out.println("2.增加学生---改进设计(输入、输出与业务逻辑相分离)");
		System.out.println("3.退出");
		System.out.println("========");
		System.out.println("please input the choice:");
	}

}

/**
 *包含业务逻辑的代码
 */
class StudentManagement{
	public static Student createStudent(String name, int grade) {
		//包含一些判断name是否已经存在之类的业务逻辑
		return new Student(name, grade);
	}
}

/**
 * 实体类
 * @author zhrb
 *
 */
class Student{
	private String name;
	private int grade;
	public Student(){}
	
	public Student(String name, int grade) {
		super();
		this.name = name;
		this.grade = grade;
	}

	public String getName() {
		return name;
	}
	public void setName(String name) {
		this.name = name;
	}
	public int getGrade() {
		return grade;
	}
	public void setGrade(int grade) {
		this.grade = grade;
	}

	@Override
	public String toString() {
		return "Student [name=" + name + ", grade=" + grade + "]";
	}
	
}
